library(ggtern)
library(ggplot2)
library(MixSim)
library(mixpack)
library(dplyr)
library(compositions)
library(latex2exp)
library(Rmixmod)

set.seed(1)

# Fixing parameters
NDATA = 500
SEP = 20
ms = list(Pi = c(1,1,1,1.5)/4.5,
          Mu = array(c(0,SEP,SEP/2,SEP/2,
                       0,  0,SEP*sqrt(3)/2,SEP*sqrt(3)/2), c(4,2)),
          S  = array(c(10, 0,
                       0, 10,
                       10, 0,
                       0, 10,
                       10, 0,
                       0, 10,
                       1, 0,
                       0, 50), c(2,2,4)))

SIM = simdataset(n=NDATA, Pi=ms$Pi, Mu=ms$Mu, S=ms$S)

df = data.frame(SIM$X)

plot(df)


mixt = mixmodCluster(df, nbCluster=4, strategy=mixmodStrategy(nbTry = 1, seed = 1),
                     models = mixmodGaussianModel(listModels="Gaussian_pk_Lk_Ck"))


xrange = round(1.1*range(df$X1))
yrange = round(1.1*range(df$X2))


xlimits = seq(-30,130, 0.5)
ylimits = seq(-30,130, 0.5)
cm = expand.grid(X1 = xlimits, X2 = ylimits)

cm$z = dmixnorm(cm[,1:2], Pi = ms$Pi, Mu = t(ms$Mu), S = ms$S)
cm$z1 = ms$Pi[1] * mvtnorm::dmvnorm(cm[,1:2], mean = ms$Mu[1,], sigma = ms$S[,,1])
cm$z2 = ms$Pi[2] * mvtnorm::dmvnorm(cm[,1:2], mean = ms$Mu[2,], sigma = ms$S[,,2])
cm$z3 = ms$Pi[3] * mvtnorm::dmvnorm(cm[,1:2], mean = ms$Mu[3,], sigma = ms$S[,,3])
cm$z4 = ms$Pi[4] * mvtnorm::dmvnorm(cm[,1:2], mean = ms$Mu[4,], sigma = ms$S[,,4])
cm$z34 = sum(ms$Pi[3:4]) * dmixnorm(cm[,1:2], Pi = ms$Pi[3:4], Mu = t(ms$Mu)[,3:4], S = ms$S[,,3:4])


step_merging = function(post, omega, lambda, f_omega = NULL, f_lambda = NULL){
  S.values = merge_step(post, omega, lambda, f_omega, f_lambda)
  
  S.values = S.values + diag(-Inf, NROW(S.values))
  ind = which(S.values == max(S.values), arr.ind = TRUE)
  new_post = merge_components(post, ind[1], ind[2])
  list('S.values' = S.values, 'post' = new_post)
}
L = list()
L[[1]] = step_merging(mixt@bestResult@proba, omega = 'prop', lambda = 'coda.norm')

POST = as.data.frame(L[[1]]$post) %>% tbl_df
POST4 = as.data.frame(mixt@bestResult@proba) %>% tbl_df


df = df %>%
  dplyr::mutate(
    Cluster = sprintf('g%d',apply(POST, 1, which.max)),
    Cluster4 = sprintf('g%d',apply(POST4, 1, which.max)))



aPOST = as.data.frame(scale(acomp(POST)))

ggplot.theme = ggplot2::theme_bw() +
  ggplot2::theme(
    axis.title = element_text(size=17),
    legend.position = 'none')
p1a <- ggplot() + 
  geom_point(data=df, aes(x=X1, y=X2, fill=Cluster4, shape=Cluster4), colour='black', size=2.2) +
  stat_contour(data=cm, aes(x=X1, y=X2, z=z4, col='g2'), linejoin='mitre', alpha=0.9, size=0.5, linetype=5) +
  stat_contour(data=cm, aes(x=X1, y=X2, z=z3, col='g4'), linejoin='mitre', alpha=0.9, size=0.5, linetype=5) +
  stat_contour(data=cm, aes(x=X1, y=X2, z=z1, col='g1'), linejoin='mitre', alpha=0.9, size=0.5, linetype=5) +
  stat_contour(data=cm, aes(x=X1, y=X2, z=z2, col='g3'), linejoin='mitre', alpha=0.9, size=0.5, linetype=5) +
  coord_fixed(xlim=xrange, ylim=yrange) + 
  scale_shape_manual(values = c(21,22,24,25)) +
  theme_bw() +
  xlab(TeX('x_1')) + ylab(TeX('x_2')) + 
  ggtitle('Sample') + ggplot.theme
p1a

p1b <- ggplot() + 
  geom_point(data=df, aes(x=X1, y=X2, fill=Cluster, shape=Cluster), colour='black', size=2.2) +
  stat_contour(data=cm, aes(x=X1, y=X2, z=z34, col='g2'), linejoin='mitre', alpha=0.9, size=0.5, linetype=5) +
  stat_contour(data=cm, aes(x=X1, y=X2, z=z1, col='g1'), linejoin='mitre', alpha=0.9, size=0.5, linetype=5) +
  stat_contour(data=cm, aes(x=X1, y=X2, z=z2, col='g3'), linejoin='mitre', alpha=0.9, size=0.5, linetype=5) +
  coord_fixed(xlim=xrange, ylim=yrange) + 
  scale_shape_manual(values = c(21,22,24)) +
  theme_bw() +
  xlab(TeX('x_1')) + ylab(TeX('x_2')) + 
  ggtitle('Sample') + ggplot.theme
p1b

## Ternary plot
ggtern.theme = ggtern::theme_bw() + 
  ggtern::theme(
    tern.axis.title = element_text(size=17),
    legend.position = 'none')

POST = POST %>%
  mutate(
    Cluster = sprintf('g%d',apply(L[[1]]$post, 1, which.max)))
p2b <- ggtern() +
  geom_mask() +
  geom_point(data=POST, aes(x = V1, y = V2, z = V3, fill=Cluster, shape=Cluster), colour='black', size=2.2) +
  scale_shape_manual(values = c(21,22,24)) +
  ggtitle('Posterior probabilities') + 
  Tlab(expression('tau[2]')) +
  Llab(expression('tau[1]')) +
  Rlab(expression('tau[3]')) + ggtern.theme
p2b


POST4 = POST4 %>%
  mutate(
    Cluster = sprintf('g%d',apply(mixt@bestResult@proba, 1, which.max)))

# Figure p2a
# Plot quaternary diagram using CoDaPack http://imae.udg.edu/codapack
# Save data to a local file:
# write.table(POST4, file='posterior4.csv', row.names = FALSE, sep = ';')

# To compute the index V
vertex = function(X){
  C = apply(X, 1, which.max)
  MEAN = sapply(1:ncol(X), function(i){
    C_i = which(C == i)
    X_i = X[C_i,]
    
    mean(apply(apply(X_i[,-i], 2, function(y, x){
      log(x) - log(y)
    }, X_i[[i]]), 1, min))

  })
  c(MEAN, 'final' = min(MEAN))
}
vertex(POST4[,1:4])
vertex(POST[,1:3])

# To calculate the methods based on distance fpc package is used
library(fpc)

Adist = dist(acomp(POST4[1:4]))
cs = cluster.stats(Adist, apply(POST4[1:4], 1, which.max), G2 = T, G3 = F)
cs[c('ch', 'g2')]

Adist = dist(acomp(POST[1:3]))
cs = cluster.stats(Adist, apply(POST[1:3], 1, which.max), G2 = T, G3 = F)
cs[c('ch', 'g2')]
