NDATA = 500
set.seed(1)

library(MixSim)
SEP = 40
ms = list(Pi = rep(1/6, 6),
          Mu = array(c(0,  0,SEP,SEP,SEP,   0,
                       0,  0,SEP,SEP,  0, SEP), c(6,2)),
          S  = array(c(50, 0,
                       0, 5,
                       5, 0,
                       0, 50), c(2,2,6)))

SIM = simdataset(n=NDATA, Pi=ms$Pi, Mu=ms$Mu, S=ms$S)

df = data.frame(SIM$X)

library(Rmixmod)
mixt = mixmodCluster(df, nbCluster=6, strategy=mixmodStrategy(nbTry = 1, seed = 1),
                     models = mixmodGaussianModel(listModels="Gaussian_pk_Lk_Ck"))
ord = c(1,6,4,2,5,3)
P = ms$Pi
M = ms$Mu[ord,]
S = lapply(ord, function(i) ms$S[,,i])

library(ggplot2)
library(latex2exp)

xrange = round(1.1*range(df$X1))
yrange = round(1.1*range(df$X2))
p <- ggplot() + 
    geom_point(data=df, aes(x=X1, y=X2), col='gray') + 
    geom_point(data=data.frame(M), aes(x=X1, y=X2), col = 'black', shape = 3, size=7) + 
    coord_fixed(xlim=xrange, ylim=yrange) + 
    ggplot2::theme_classic() +
    xlab(TeX('x_1')) + ylab(TeX('x_2'))



xlimits = seq(-30,130, 0.5)
ylimits = seq(-30,130, 0.5)
cm = expand.grid(X1 = xlimits, X2 = ylimits)

library(mixpack)
cm$z = dmixnorm_solution(cm, solution=mixt)

p.c6 <- p + stat_contour(data=cm, aes(x=X1, y=X2, z=z), col='blue')  +
  geom_point(data=data.frame(M), aes(x=X1, y=X2), col = 'black', shape = 3, size=7)
p.c6
