NDATA = 500
set.seed(1)

library(MixSim)
SEP = 40
ms = list(Pi = rep(1/6, 6),
          Mu = array(c(0,  0,SEP,SEP,SEP,   0,
                       0,  0,SEP,SEP,  0, SEP), c(6,2)),
          S  = array(c(50, 0,
                       0, 5,
                       5, 0,
                       0, 50), c(2,2,6)))

SIM = simdataset(n=NDATA, Pi=ms$Pi, Mu=ms$Mu, S=ms$S)

df = data.frame(SIM$X)

library(Rmixmod)
mixt = mixmodCluster(df, nbCluster=6, strategy=mixmodStrategy(nbTry = 1, seed = 1),
                     models = mixmodGaussianModel(listModels="Gaussian_pk_Lk_Ck"))

P = ms$Pi
M = ms$Mu
S = lapply(1:6, function(i) ms$S[,,i])




library(ggplot2)
library(latex2exp)

xrange = round(1.1*range(df$X1))
yrange = round(1.1*range(df$X2))




xlimits = seq(-30,130, 0.5)
ylimits = seq(-30,130, 0.5)
cm = expand.grid(X1 = xlimits, X2 = ylimits)
library(mixpack)
cm$z = dmixnorm_solution(cm, solution=mixt)



library(dplyr)
library(mvtnorm)

HP3a = get_hierarchical_partition(mixt@bestResult@proba, omega = 'cnst', lambda = 'entr')
HP = lapply(HP3a, lapply, function(v) (1:6)[v])
partition = HP[[4]]

CN4 = lapply(partition, function(part){
  cn = expand.grid(X1 = xlimits, X2 = ylimits)
  cn$z = dmixnorm_solution(cn, mixt, part = part) #dmvnorm(cn, mean = Mu[,i], sigma = S[,,i])
  cn$id = sprintf('{%s}',paste(c(5,2,3,1,4,6)[part], collapse=','))
  cn
}) %>% bind_rows

ii = as.numeric(cluster_partition(mixt@bestResult@proba, setNames(partition, 1:4)))
df$id = sapply(ii, function(i) sprintf('{%s}',paste(c(5,2,3,1,4,6)[partition[[i]]], collapse=',')) )

p.cn4 <- ggplot() + 
  stat_contour(data=cm, aes(x=X1, y=X2, z=z), alpha=0.1) +
  geom_point(data=df, aes(x=X1, y=X2), alpha=0.8, size=1) +
  stat_contour(data=CN4, aes(x=X1, y=X2, z=z), col='blue', alpha=0.8) + 
  facet_wrap(~id, nrow=2) + ggplot2::theme_classic() + ggplot2::theme(legend.position="none") +
  scale_x_continuous(breaks=c(0, 20, 40)) + scale_y_continuous(breaks=c(0, 20, 40)) +
  xlab(TeX('x_1')) + ylab(TeX('x_2'))
p.cn4
