NDATA = 500
set.seed(1)

library(MixSim)
SEP = 40
ms = list(Pi = rep(1/6, 6),
          Mu = array(c(0,  0,SEP,SEP,SEP,   0,
                       0,  0,SEP,SEP,  0, SEP), c(6,2)),
          S  = array(c(50, 0,
                       0, 5,
                       5, 0,
                       0, 50), c(2,2,6)))

SIM = simdataset(n=NDATA, Pi=ms$Pi, Mu=ms$Mu, S=ms$S)

df = data.frame(SIM$X)

library(Rmixmod)
mixt = mixmodCluster(df, nbCluster=6, strategy=mixmodStrategy(nbTry = 1, seed = 1),
                     models = mixmodGaussianModel(listModels="Gaussian_pk_Lk_Ck"))



library(tidyr)
f_omega1 = function(v_tau, a) as.numeric(which.max(v_tau) == a)
f_omega2 = function(v_tau, a) v_tau[a]
f_lambda= function(v_tau, a, b) exp(-log(v_tau[a]/v_tau[b])^2)


HP1a = get_hierarchical_partition(mixt@bestResult@proba, omega = 'prop', lambda = 'demp')
HP2a = get_hierarchical_partition(mixt@bestResult@proba, omega = 'prop', lambda = 'demp.mod')
HP3a = get_hierarchical_partition(mixt@bestResult@proba, omega = 'cnst', lambda = 'entr')
HP4a = get_hierarchical_partition(mixt@bestResult@proba, omega = 'prop', lambda = 'coda')


HP1b = get_hierarchical_partition(mixt@bestResult@proba, omega = 'prop', lambda = 'prop')
HP2b = get_hierarchical_partition(mixt@bestResult@proba, f_omega = f_omega2, f_lambda = f_lambda)


df2 = data_frame(
  K = factor(2:6, rev(2:6)),
  'DEMP' = attr(HP1a, 'S.value')[1:5],
  'DEMP modified' = attr(HP2a, 'S.value')[1:5],
  #'Dif. Entropy' = attr(HP3a,'S.value')[1:5],
  #'Log-ratio' = attr(HP4a,'S.value')[1:5],
  'Proportion' = attr(HP1b, 'S.value')[1:5],
  'Aitchison (scaled)' = 1-exp(-attr(HP2b, 'S.value')[1:5]),
  'Dif. Entropy (scaled)' = attr(HP3a,'S.value')[1:5]/(-log(1/2:6)),
  'Log-ratio (scaled)' = exp(attr(HP4a, 'S.value')[1:5])/(1 + exp(attr(HP4a, 'S.value')[1:5]))
) %>% gather(key=method, value=S.value, -K)
df2$method = factor(df2$method, levels = c('DEMP', 'DEMP modified', 'Proportion', 'Aitchison (scaled)', 
                                           'Dif. Entropy (scaled)', 'Log-ratio (scaled)'))


p1 = ggplot() +
  geom_point(data=df2, aes(x=K, y=S.value), size=3) +
  facet_wrap(~method, scales = 'free', nrow=2 ) +
  ggplot2::theme_classic() +
  xlab('Clusters') + ylab('S-value') +
  ggtitle('S-value during the merging process') +
  theme(axis.line.x = element_line(color="black"),
        axis.line.y = element_line(color="black"))
p1

## Second part
library(fpc)
library(mixpack)
library(compositions)
# post = fit$posterior
# post = mixt@bestResult@proba
## Merging
step_merging = function(post, omega, lambda, f_omega = NULL, f_lambda = NULL){
  S.values = merge_step(post, omega, lambda, f_omega, f_lambda)
  
  S.values = S.values + diag(-Inf, NROW(S.values))
  ind = which(S.values == max(S.values), arr.ind = TRUE)
  new_post = merge_components(post, ind[1], ind[2])
  list('S.values' = S.values, 'post' = new_post)
}

L.post = function(post, omega, lambda){
  L = list()
  L[[1]] = list('S.value' = NA, 'post' = post)
  for(i in 2:NCOL(post)){
    L[[i]] = step_merging(L[[i-1]]$post, omega, lambda)
  }
  L
}

c.stats.par = function(post, par, post0 = NULL){
  if(is.null(post0)){
    post0 = post
  }
  if(NCOL(post) == 1){
    return(NA)
  }
  post0[post0 == 0] = .Machine$double.xmin
  apost = acomp(post0)
  Adist = dist(apost)
  if(par == 'g2'){
    c.stats = cluster.stats(Adist, apply(post, 1, which.max), G2 = T, G3 = F)
  }else{
    c.stats = cluster.stats(Adist, apply(post, 1, which.max), G2 = F, G3 = F)
  }
  
  c.stats[[par]]
}

post = mixt@bestResult@proba
L = L.post(post = post, omega = 'prop', lambda = 'coda.norm')

L.index = function(L, index, post0){
  sapply(L, function(l) c.stats.par(l$post, index, post0 = post0))
}

INDEXS = c('cluster.number' = 'K', 
           'average.between' = 'Average Between', 
           'average.within' = 'Average Within', 
           'ch' = 'Calinski-Harabasz (G1)', 
           #            'pearsongamma' = 'Normalized gamma', 
           #            'dunn' = 'Dunn', 
           #            'dunn2' = 'Dunn2', 
           #            'avg.silwidth' = 'Average silouhette', 
           #            'sindex' = 'Separation',
           'g2' = 'Goodman-Kruskal (G2)')
#V.index2 = lapply(names(INDEXS), function(index) L.index(L, index, L[[1]]$post)) %>% as.data.frame %>% tbl_df
#names(V.index2) = INDEXS

V.index = lapply(names(INDEXS), function(index) L.index(L, index, NULL)) %>% as.data.frame %>% tbl_df
names(V.index) = INDEXS

## Time consuming
V.index[['Line']] = sapply(L[1:6], function(l){
  if( NCOL(l$post) == 1) return(NA)
  X = data.frame(l$post) %>% tbl_df
  C = apply(X, 1, which.max)
  
  MEAN = sapply(1:NCOL(l$post), function(i){
    C_i = which(C == i)
    X_i = X[C_i,]
    
    mean( log(X_i[[i]]) - log(apply(X_i[,-i], 1, sum)) )
  })
  min(MEAN)
})

V.index[['Vertex']] = sapply(L[1:6], function(l){
  if( NCOL(l$post) == 1) return(NA)
  X = data.frame(l$post) %>% tbl_df
  C = apply(X, 1, which.max)
  
  MEAN = sapply(1:NCOL(l$post), function(i){
    C_i = which(C == i)
    X_i = X[C_i,]
    
    min(apply(X_i[,-i], 2, function(y, x){
      median(log(x) - log(y))
    }, X_i[[i]]))
  })
  min(MEAN)
})

df = V.index %>% select(c(1,4,5,7))  %>% na.omit %>% mutate(
  K = factor(K, NROW(V.index):1)
) %>% gather(key = index, value = value, -K)

ggplot() +
  #geom_line(data=df, aes(x=K, y=value), group=1) +
  geom_point(data=df, aes(x=K, y=value), size=3) +
  facet_wrap(~index, scales='free', nrow=1) + 
  ggplot2::theme_classic() +
  ylab('Statistic value') + xlab('Clusters') +
  ggtitle('Posteriori probability distance based heuristics')
