library(zCompositions)
library(mixtools)
library(dplyr)
library(tidyr)
library(ggplot2)
library(mixpack)

data("Pigs")

pigs <- Pigs %>% tbl_df

bic = function(loglik, k, n, d) -2 * loglik + (k * d - 1) * log(n)

set.seed(2)
K = 6
suppressMessages(
  FIT <- lapply(1:100, function(it, K){
    multmixEM(as.matrix(pigs), k = K)
  }, K)
)

fit = FIT[[which.min(sapply(FIT, function(fit) bic(fit$loglik, K, NROW(pigs), NCOL(pigs)) ))]]


d = as.data.frame(fit$theta)
d$pi = fit$lambda

hp = get_hierarchical_partition(fit$posterior, omega = 'prop', lambda = 'coda.norm')
seq_partition = lapply(hp, sapply, paste, collapse=',')

pigs.g <- pigs %>% mutate(
  TOTAL = BED+HALF.BED+PASSAGE+HALF.PASS+FEEDER+HALF.FEED,
  Comp = as.character(apply(fit$posterior, 1, which.max)))

K = 3
clusters = mixpack::cluster_partition(fit$posterior, partition = hp[[K]] %>% setNames(., sapply(., function(lbl) paste(lbl, collapse='-'))))
d = pigs.g %>%
  mutate(
    Cluster = clusters
  ) %>% group_by(Cluster, Comp) %>%
  summarise(
    BED = mean(BED/TOTAL),
    HALF.BED = mean(HALF.BED/TOTAL),
    PASSAGE = mean(PASSAGE/TOTAL),
    HALF.PASS = mean(HALF.PASS/TOTAL),
    FEEDER = mean(FEEDER/TOTAL),
    HALF.FEED = mean(HALF.FEED/TOTAL))%>%
  gather(key=Variable, value=Proportion, -Cluster, -Comp) %>%
  mutate(
    Variable = factor(Variable, levels = c('BED', 'HALF.BED', 'PASSAGE', 'HALF.PASS', 'FEEDER', 'HALF.FEED'))
  )

ggplot() +
  geom_bar(data = d, aes(x=Variable, y=Proportion, fill=Comp), color='black', stat = 'identity', position=position_dodge()) +
  facet_wrap(~Cluster, ncol=3) +
  theme_classic() +
  theme(legend.position="none",
        axis.text.x = element_text(angle = 90, hjust = 1)) + xlab("") + ylab('Relative Frequency')
