if(!exists('build')) build = 'MAXOVERLAP_0.01-NSIM_00100-NDATA_00500-K0_003-DIM_002-SEED_0078'
pattern_build = "MAXOVERLAP_(0\\.[0-9]+)-NSIM_([0-9]+)-NDATA_([0-9]+)-K0_([0-9]+)-DIM_([0-9]+)-SEED_([0-9]+)*"
cat(build,'\n')

MAXOVERLAP = as.numeric(sub(pattern_build, "\\1", build))
NSIM = as.numeric(sub(pattern_build, "\\2", build))
NDATA = as.numeric(sub(pattern_build, "\\3", build))
K0 = as.numeric(sub(pattern_build, "\\4", build))
DIM = as.numeric(sub(pattern_build, "\\5", build))
SEED = as.numeric(sub(pattern_build, "\\6", build))


# The following script generates `NSIM` simulations each one containing:
# - A dataset of dimension `DIM` with `NDATA` observations generated from a mixture with `K0` *non-spherical*
# components with max overlapping `MAXOVERLAP`
# - The component from which each observation was generated (a number between 1 and K0)
# - The posterior probability after adjusting a mixture with Kf *spherical* components to the data

library(MixSim)
library(dplyr)
library(mclust)

set.seed(SEED)
simulation = lapply(1:NSIM, function(i){
  ms = MixSim(MaxOmega=MAXOVERLAP, K=K0, p=DIM, PiLow=0.01, sph=FALSE)
  sim = simdataset(n=NDATA, Pi=ms$Pi, Mu=ms$Mu, S=ms$S)
  
  m = Mclust(sim$X, G=1:20, modelNames ="VII")
  tau = m$z
  tau[tau == 0] = .Machine$double.xmin  
  list('data' = sim$X, 'g' = sim$id, 'tau' = tau, 'mclust' = m)
})
save(simulation, file=sprintf('data/dataset-%s.RData', build))

