library(fpc)
library(mixpack)
library(compositions)
library(dplyr)
library(tidyr)
library(ggplot2)

if(!exists('build')){
  build = 'MAXOVERLAP_0.25-NSIM_00100-NDATA_00500-K0_003-DIM_002-SEED_0001'
}
pattern_build = "MAXOVERLAP_(0\\.[0-9]+)-NSIM_([0-9]+)-NDATA_([0-9]+)-K0_([0-9]+)-DIM_([0-9]+)-SEED_([0-9]+)*"

MAXOVERLAP = as.numeric(sub(pattern_build, "\\1", build))
NSIM = as.numeric(sub(pattern_build, "\\2", build))
NDATA = as.numeric(sub(pattern_build, "\\3", build))
K0 = as.numeric(sub(pattern_build, "\\4", build))
DIM = as.numeric(sub(pattern_build, "\\5", build))
SEED = as.numeric(sub(pattern_build, "\\6", build))

load(sprintf('data/dataset-%s.RData', build))

SIMULATION = simulation[[1]]
post = SIMULATION[[3]]
rm(simulation)

## Merging
step_merging = function(post, omega, lambda, f_omega = NULL, f_lambda = NULL){
  S.values = merge_step(post, omega, lambda, f_omega, f_lambda)
  
  S.values = S.values + diag(-Inf, NROW(S.values))
  ind = which(S.values == max(S.values), arr.ind = TRUE)
  new_post = merge_components(post, ind[1], ind[2])
  list('S.values' = S.values, 'post' = new_post)
}

L.post = function(post, omega, lambda){
  L = list()
  L[[1]] = list('S.value' = NA, 'post' = post)
  for(i in 2:NCOL(post)){
    L[[i]] = step_merging(L[[i-1]]$post, omega, lambda)
  }
  L
}

c.stats.par = function(post, par, post0 = NULL){
  if(is.null(post0)){
    post0 = post
  }
  if(NCOL(post) == 1){
    return(NA)
  }
  post0[post0 == 0] = .Machine$double.xmin
  apost = acomp(post0)
  Adist = dist(apost)
  if(par == 'g2'){
    c.stats = cluster.stats(Adist, apply(post, 1, which.max), G2 = T, G3 = F)
  }else{
    c.stats = cluster.stats(Adist, apply(post, 1, which.max), G2 = F, G3 = F)
  }
  
  c.stats[[par]]
}

L = L.post(post = post, omega = 'prop', lambda = 'coda.norm')

L.index = function(L, index, post0){
  sapply(L, function(l) c.stats.par(l$post, index, post0 = post0))
}

INDEXS = c('cluster.number' = 'K', 
           'average.between' = 'Average Between', 
           'average.within' = 'Average Within', 
           'ch' = 'Calinski-Harabasz (G1)', 
           #            'pearsongamma' = 'Normalized gamma', 
           #            'dunn' = 'Dunn', 
           #            'dunn2' = 'Dunn2', 
           #            'avg.silwidth' = 'Average silouhette', 
           #            'sindex' = 'Separation',
           'g2' = 'Goodman-Kruskal (G2)')
#V.index2 = lapply(names(INDEXS), function(index) L.index(L, index, L[[1]]$post)) %>% as.data.frame %>% tbl_df
#names(V.index2) = INDEXS

# Time-consuming operation (5 minutes more or less in a laptop)
V.index = lapply(names(INDEXS), function(index) L.index(L, index, NULL)) %>% as.data.frame %>% tbl_df
names(V.index) = INDEXS

V.index[['Line']] = sapply(L[1:SIMULATION[[4]]$G], function(l){
  if( NCOL(l$post) == 1) return(NA)
  X = data.frame(l$post) %>% tbl_df
  C = apply(X, 1, which.max)
  
  MEAN = sapply(1:NCOL(l$post), function(i){
    C_i = which(C == i)
    X_i = X[C_i,]
    
    mean( log(X_i[[i]]) - log(apply(X_i[,-i], 1, sum)) )
  })
  min(MEAN)
})
V.index[['Vertex']] = sapply(L[1:SIMULATION[[4]]$G], function(l){
  if( NCOL(l$post) == 1) return(NA)
  X = data.frame(l$post) %>% tbl_df
  C = apply(X, 1, which.max)
  
  MEAN = sapply(1:NCOL(l$post), function(i){
    C_i = which(C == i)
    X_i = X[C_i,]
    
    min(apply(X_i[,-i], 2, function(y, x){
      median(log(x) - log(y))
    }, X_i[[i]]))
  })
  min(MEAN)
})

df = V.index %>% select(c(1,4,5,7))  %>% na.omit %>% mutate(
  K = factor(K, NROW(V.index):1)
) %>% gather(key = index, value = value, -K)


############
HP1a = get_hierarchical_partition(post, omega = 'prop', lambda = 'demp')
HP2a = get_hierarchical_partition(post, omega = 'prop', lambda = 'demp.mod')
HP3a = get_hierarchical_partition(post, omega = 'cnst', lambda = 'entr')
HP4a = get_hierarchical_partition(post, omega = 'prop', lambda = 'coda')


HP1b = get_hierarchical_partition(post, omega = 'prop', lambda = 'prop')
f_omega2 = function(v_tau, a) v_tau[a]
f_lambda= function(v_tau, a, b) exp(-log(v_tau[a]/v_tau[b])^2)
HP2b = get_hierarchical_partition(post, f_omega = f_omega2, f_lambda = f_lambda)


df2 = data_frame(
  K = factor(2:6, rev(2:6)),
  'DEMP' = attr(HP1a, 'S.value')[1:5],
  'DEMP modified' = attr(HP2a, 'S.value')[1:5],
  #'Dif. Entropy' = attr(HP3a,'S.value')[1:5],
  #'Log-ratio' = attr(HP4a,'S.value')[1:5],
  'Proportion' = attr(HP1b, 'S.value')[1:5],
  'Aitchison (scaled)' = 1-exp(-attr(HP2b, 'S.value')[1:5]),
  'Dif. Entropy (scaled)' = attr(HP3a,'S.value')[1:5]/(-log(1/2:6)),
  'Log-ratio (scaled)' = exp(attr(HP4a, 'S.value')[1:5])/(1 + exp(attr(HP4a, 'S.value')[1:5]))
) %>% gather(key=method, value=S.value, -K)
df2$method = factor(df2$method, levels = c('DEMP', 'DEMP modified', 'Proportion', 'Aitchison (scaled)', 
                                           'Dif. Entropy (scaled)', 'Log-ratio (scaled)'))

save(list = ls(), file = sprintf('data/results_visualisation-%s.RData', build))
