library(mixpack)
library(MixSim)
library(dplyr)

if(!exists('build')) build = 'MAXOVERLAP_0.01-NSIM_00100-NDATA_00500-K0_003-DIM_006-SEED_0001'

pattern_build = "MAXOVERLAP_(0\\.[0-9]+)-NSIM_([0-9]+)-NDATA_([0-9]+)-K0_([0-9]+)-DIM_([0-9]+)-SEED_([0-9]+)*"
MAXOVERLAP = as.numeric(sub(pattern_build, "\\1", build))
NSIM = as.numeric(sub(pattern_build, "\\2", build))
NDATA = as.numeric(sub(pattern_build, "\\3", build))
K0 = as.numeric(sub(pattern_build, "\\4", build))
DIM = as.numeric(sub(pattern_build, "\\5", build))
SEED = as.numeric(sub(pattern_build, "\\6", build))

load(file=sprintf('data/dataset-%s.RData', build))
load(file=sprintf('data/rnd_hpartitions-%s.RData', build))

df.eval = bind_rows(lapply(1:NSIM, function(i){
  TAU = simulation[[i]]$tau
  Ki = length(hpartitions[[i]])
  if(Ki >= K0){
    HPART = hpartitions[[i]][[K0]]
    hp_clust = apply(sapply(HPART, function(ind) rowSums(matrix(TAU[,ind], ncol=length(ind)) )), 1, which.max)
    RES = data.frame(RandIndex(simulation[[i]]$g, hp_clust))
    RES$Ki = Ki
  }else{
    RES = data.frame('R' = NA, 'AR' = NA, 'F' = NA, 'M' = NA, 'Ki' = Ki)
  }
  RES
}))

save(df.eval, file=sprintf('data/rnd_evaluate-%s.RData', build))
